/*
 *  Copyright (C) 2002 Tomasz Kojm <zolw@konarski.edu.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "readdb.h"
#include "others.h"
#include "strings.h"
#include "shared.h"
#include "matcher.h"

int parse_add(struct node *root, const char *virname, const char *hexstr)
{
	struct patt *new;

    /* decode hexstring and prepare new entry */

    new = (struct patt *) mcalloc(1, sizeof(struct patt));
    new->length = strlen(hexstr) / 2;

    if(new->length > root->maxpatlen)
	root->maxpatlen = new->length;

    if((new->pattern = hex2str(hexstr)) == NULL) {
	free(new);
	return 1;
    }

    new->virname = mcalloc(strlen(virname) + 1, sizeof(char));
    strncpy(new->virname, virname, strlen(virname));

    addpatt(root, new);

    return 0;
}

/* this functions returns a pointer to the root of trie */

struct node *readdb(const char *filename)
{
	FILE *fd;
	char buffer[BUFFSIZE], *pt, *start;
	int line = 0;
	struct node *trie;

    if((fd = fopen(filename, "rb")) == NULL) {
	mprintf("!Can't open file %s\n", filename);
	return NULL;
    }

    trie = (struct node *) mcalloc(1, sizeof(struct node));
    trie->maxpatlen = 0;

    while(fgets(buffer, BUFFSIZE, fd)) {
	line++;
	chomp(buffer);

	pt = strchr(buffer, '=');
	if(!pt) {
	    mprintf("!Malformed pattern line %d.\n", line);
	    return NULL;
	}

	start = buffer;
	*pt++ = 0;

	if(*pt == '=') continue;

	claminfo.signs++;
	if(parse_add(trie, start, pt)) return NULL;
    }

    fclose(fd);

#ifdef PRINT_SIG
    print_signature_list(sigtab);
    exit(0);
#endif

    return trie;
}

#ifdef PRINT_SIG
void print_signature_list(struct signature **sigtab)
{
	int i, j;
	struct signature *list;
	FILE *fd;

    fd = fopen("DecodeDB", "wb");

    for(i = 0; i < ALL; i++) {
	list = sigtab[i]->next;

	while(list) {
	    mprintf("\nVIRUS NAME: %s\nSIGNATURE: ", list->virname);
	    fwrite(list->pattern, 1, list->pattlen, fd);
	    fprintf(fd, "\n");
	    list = list->next;
	}
    }

    fclose(fd);
}
#endif
